
import UIKit
import SafariServices
import NCPlayerSDK


class ViewController: UIViewController, NCPlayerDelegate, SFSafariViewControllerDelegate {
    
    @IBOutlet weak var currentTimeText: UITextField!
    @IBOutlet weak var playerView: NCPlayerView?
    var isFullscreen = false
    var safariViewController: SFSafariViewController?
        
    

    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        //라이선스키
        let licenseKey: String = "a624346b30f93b892c4fa2f636a57c83" //demo용 라이선스키입니다. 실제키로 변경이 필요합니다.
        let enableCaptureProtect = true // 스크린 캡쳐 및 화면녹화 방지 여부
        
        // SDK 초기화
        NCPlayer.shared.initialize(licenseKey: licenseKey, enableCaptureProtect: enableCaptureProtect)
        NCPlayer.shared.delegate = self
      
        
        // case 1 서버에서 플레이어 설정 및 재생 설정 -------
        
        // 플레이 예제 : https://vpe.sgrsoft.com/api/playUrl
        // One Click Multi DRM : https://vpe.sgrsoft.com/api/drmTest
        // Pallycon DRM : https://vpe.sgrsoft.com/api/drmTestPallycon
        
        // FairPlay DRM은 반드시 서버설정(case 1) 방식으로 필요
        
        
        // 서버에서 playerOptions 가져오기 예제
        fetchPlayerOptions("https://vpe.sgrsoft.com/api/playUrl")
        
        
        
        // case 2 플레이어에서 직접 옵션 및 재생 설정 -------
        //플레이어 옵션 (콘솔에서 만들어진 스크립트 옵션을 사용)
 /*       let playerOptions = """
         {
            "playlist":[
                {
                       "file":"https://m4qgahqg2249.edge.naverncp.com/hls/a4oif2oPHP-HlGGWOFm29A__/endpoint/sample/221027_NAVER_Cloud_intro_Long_ver_AVC_,FHD_2Pass_30fps,HD_2Pass_30fps,SD_2Pass_30fps,.mp4.smil/master.m3u8",
                       "poster":"https://2ardrvaj2252.edge.naverncp.com/endpoint/sample/221027_NAVER_Cloud_intro_Long_ver_01.jpg",
                       "description":{
                           "title":"네이버클라우드 테스트 영상",
                           "created_at":"2024.07.13",
                           "profile_name":"네이버클라우드",
                           "profile_image":"https://player.vpe.naverncp.com/images/d127c8db642716d84b3201f1d152e52a.png"
                       }
                },
            ],
            "autostart":true,
            "muted":true,
            "progressBarColor":"#ff0000",
            "aspectRatio":"16/9",
         }
         
         """
        
        playerView?.play(playerOptions)
  
  */
        
        
    }
    
    func fetchPlayerOptions(_ setUrl: String) {
          // JSON 서버 URL 설정
          guard let url = URL(string: setUrl) else {
              print("유효하지 않은 URL")
              return
          }
            
            // URL 세션 데이터 태스크 생성
            let task = URLSession.shared.dataTask(with: url) { [weak self] data, response, error in
                guard let self = self else { return }
                if let error = error {
                    print("playerOptions 가져오기 실패: \(error)")
                    return
                }
                
                guard let data = data else {
                    print("데이터를 받지 못했습니다")
                    return
                }
                
          
                // JSON 데이터를 문자열로 변환
                if let jsonString = String(data: data, encoding: .utf8) {
                    DispatchQueue.main.async {
                        // 가져온 옵션으로 비디오 재생 시작
                        self.playerView?.play(jsonString)
                    }
                } else {
                    print("데이터를 JSON 문자열로 변환 실패")
                }

                // self.openInSafariViewController(url)
                
            }
            
            // 태스크 시작
            task.resume()
        }


    func openInSafariViewController(_ url: URL) {
            safariViewController = SFSafariViewController(url: url)
            safariViewController?.delegate = self
            safariViewController?.modalPresentationStyle = .formSheet
            if let safariVC = safariViewController {
                present(safariVC, animated: true, completion: nil)
            }
        }
        
        // SFSafariViewControllerDelegate - Safari View Controller 닫힘 처리
        func safariViewControllerDidFinish(_ controller: SFSafariViewController) {
            print("SafariViewController가 닫혔습니다.")
            safariViewController = nil
        }

    override func viewDidLoad() {
        super.viewDidLoad()
        // 뷰 터치시 키보드 내리기
        let tap = UITapGestureRecognizer(target: view, action: #selector(UIView.endEditing))
        view.addGestureRecognizer(tap)
    }


    @IBOutlet weak var tfAccessKey: UITextField!
    
    @IBAction func applyAuthInformation(_ sender: Any) {
        NCPlayer.shared.licenseKey = tfAccessKey.text
        playerView?.applySettings()
    }
    
    @IBAction func switchEnvironment(_ sender: Any) {
        let segment = sender as! UISegmentedControl
        NCPlayer.shared.environment = ["production", "beta"][segment.selectedSegmentIndex]
        playerView?.applySettings()
    }
    
    @IBAction func onEditEnded(_ sender: Any) {
        (sender as? UITextField)?.resignFirstResponder()
    }

    //스테이터스바 색상 처리
    override var preferredStatusBarStyle: UIStatusBarStyle {
        return .lightContent
    }
    
    //네트워크 상태 여부
    func NCPlayerNetworkStatus(state: String) {
        print("networkStatus: \(state)")
    }
    
    //가로모드 상태 여부
    func NCPlayerIsLandscape(state: Bool) {
        print("isLandscape: \(state)")
        if state {
            currentOrientationMask = .landscape
            currentOrientation = UIDevice.current.orientation == .landscapeLeft ? .landscapeRight : .landscapeLeft
            
            
        }else{
            currentOrientationMask = .portrait
            currentOrientation = .portrait
        }
        
        applyOrientation(orientation: currentOrientation)
        
    }
    //캡쳐 여부
    func NCPlayerIsCaptured(state: Bool) {
        print("isCaptured: \(state)")
    }
    //녹화됨
    func NCPlayerIsRecording() {
        print("isRecording")
    }
    
    //현재 재생시간이 변경될 때 발생하는 이벤트
    func NCPlayerOnTimeUpdate(data: Dictionary<String, Any>?) {
//        if let data = data {
//            print("NCPlayerOnTimeUpdate: ",data)
//        }
    }
    
    //이전 영상으로 넘어갈때 발생하는 이벤트
    func NCPlayerOnPrevTrack(data: Dictionary<String, Any>?) {
        if let data = data {
            print("NCPlayerOnPrevTrack: ",data)
        }
    }
    
    //다음 영상으로 넘어갈때 발생하는 이벤트
    func NCPlayerOnNextTrack(data: Dictionary<String, Any>?) {
        if let data = data {
            print("NCPlayerOnNextTrack: ",data)
        }
    }
    
    //컨트롤바 활성화 or 비활성화 여부
    func NCPlayerOnControlbarActive(active: Bool) {
        print("NCPlayerOnControlbarActive: ",active)
    }
    
    //video 재생 요청이 성공하였을 때 발생하는 이벤트
    func NCPlayerOnPlay(data: Dictionary<String, Any>?) {
//
//        let description = data?["description"] as? Dictionary<String, Any>
//        let title = description?["title"] as! String
//        let artist = description?["profile_name"] as! String
//        let poster = data?["poster"] as? String

       print("NCPlayerOnPlay: play")
    }
    
    //video가 일시정지 되었을때 발생하는 이벤트
    func NCPlayerOnPause() {
        print("NCPlayerOnPause: pause")
    }
    
    //재생이 끝까지 완료되었을 때 발생하는 이벤트
    func NCPlayerOnEnd() {
        print("NCPlayerOnEnd: end")
    }
    
    func NCPlayerOnFullscreenOff() {
        isFullscreen = false
        // 풀스크린이 해제되면 플레이어 사이즈는 16:9 사이즈로
        playerHeightConstaint = playerHeightConstaint!.applyAspectRatio(view: playerView!, ratio: 16.0/9.0)

    }
    //home indicator 없애기
    override var prefersHomeIndicatorAutoHidden: Bool {
        return isFullscreen
    }
    
    @IBOutlet weak var playerHeightConstaint: NSLayoutConstraint?
    
    func NCPlayerOnFullscreenOn() {
        isFullscreen = true

        // 아이패드일 경우 풀스크린 화면은 4:3 비율로 조정
        if UIDevice.current.userInterfaceIdiom == .pad {
            playerHeightConstaint = playerHeightConstaint!.applyAspectRatio(view: playerView!, ratio: 4.0/3.0)
        }


    }
    
    func NCPlayerOnFullscreen(state: Bool) {
        print("NCPlayerOnFullscreen: ",state)
    }

    var currentOrientationMask: UIInterfaceOrientationMask = .portrait
    var currentOrientation: UIInterfaceOrientation = .portrait
    
    func NCPlayerRequestOrientation(orientation: String) {
        
        print("NCPlayerRequestOrientation: ", orientation)
        if (orientation == "landscape") {
            currentOrientationMask = .landscape
            currentOrientation = .landscapeRight

        } else {
            currentOrientationMask = .portrait
            currentOrientation = .portrait
        }
        
        applyOrientation(orientation: currentOrientation)
        
    }
    
    override var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        return currentOrientationMask;
    }
    
    override var preferredInterfaceOrientationForPresentation: UIInterfaceOrientation {
        return currentOrientation;
    }
    
    override var prefersStatusBarHidden: Bool {
        return currentOrientation.isLandscape
    }

    @IBAction func prev(_ sender: Any) {
        playerView?.prev()
    }

    @IBAction func next(_ sender: Any) {
        playerView?.next()
    }

    @IBAction func play(_ sender: Any) {
        playerView?.videoPlay()
    }

    @IBAction func pause(_ sender: Any) {
        playerView?.pause()
    }

    @IBAction func mute(_ sender: Any) {
        playerView?.mute()
    }

    @IBAction func controlbarActive(_ sender: Any) {
        playerView?.controlBarActive()

    }

    @IBAction func controlBarDeactive(_ sender: Any) {
        playerView?.controlBarDeactive()
    }

    @IBAction func currentTimeChange(_ sender: Any) {
        playerView?.currentTime(currentTimeText.text ?? "0")
    }

    @IBAction func releasePlayerView(_ sender: Any) {
        playerView?.removeFromSuperview()
        playerView = nil
    }
}

extension NSLayoutConstraint {
    
    func applyAspectRatio(view: UIView, ratio: CGFloat) -> NSLayoutConstraint {
        let newConstraint = NSLayoutConstraint(item: self.firstItem!, attribute: self.firstAttribute, relatedBy: self.relation, toItem: self.secondItem, attribute: self.secondAttribute, multiplier: ratio, constant: self.constant)
        view.removeConstraint(self)
        view.addConstraint(newConstraint)
        view.layoutIfNeeded()
        return newConstraint
    }
}
